import {Component, OnInit} from '@angular/core';
import {EditorComponent} from "ngx-monaco-editor-v2";
import {FormsModule} from "@angular/forms";
import {InputOutputComponent} from "../input-output/input-output.component";
import {JsonTransformService} from "../json-transform.service";
import {
  DebounceTime,
  GenerateDefaultJsonObjectString,
  MonacoJsonConfig,
  ReadOnlyMonacoJsonConfig
} from "../defaults";
import {debounceTime, Subject} from "rxjs";

@Component({
  selector: 'app-prettify',
  standalone: true,
  imports: [
    EditorComponent,
    FormsModule,
    InputOutputComponent
  ],
  templateUrl: './prettify.component.html',
  styleUrl: './prettify.component.scss'
})
export class PrettifyComponent implements OnInit {
  input: string = GenerateDefaultJsonObjectString();
  $input: Subject<string> = new Subject<string>();
  inputOptions = MonacoJsonConfig;
  output: string = GenerateDefaultJsonObjectString(2);
  outputOptions = ReadOnlyMonacoJsonConfig;
  error: string = "";

  constructor(private service: JsonTransformService) {
  }

  ngOnInit(): void {
    this.$input
      .pipe(debounceTime(DebounceTime))
      .subscribe(input => {
        this.update(input)
      });
  }

  update(input: string): void {
    this.service
      .prettify(input)
      .subscribe({
        next: response => {
          console.log(response);
          this.output = response.body.result;
        },
        error: response => {
          console.log(response)
          if (response.status === 499) {
            this.output = response.error.detail;
            console.log(response.error.detail);
          }
        }
      });
  }

  handleInputChange($event: any): void {
    console.log($event);
    this.$input.next($event);
  }
}
